@@ -4,7 +4,6 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
-	"io"
 	"log"
 	"os"
 	"os/exec"
@@ -13,7 +12,6 @@ import (
 	"sort"
 	"strconv"
 	"strings"
-	"syscall"
 	"text/template"
 	"time"
 
@@ -22,6 +20,13 @@ import (
 	"gopkg.in/yaml.v3"
 )
 
+const (
+	Version               = "v0.1.5"
+	GithubRepo            = "https://github.com/eduardoagarcia/shef"
+	PublicRecipesFilename = "recipes.tar.gz"
+	PublicRecipesFolder   = "recipes"
+)
+
 type Config struct {
 	Recipes []Recipe `yaml:"recipes"`
 }
@@ -244,7 +249,10 @@ func renderTemplate(tmplStr string, vars map[string]interface{}) (string, error)
 		return "", fmt.Errorf("template execution error: %w", err)
 	}
 
-	return buf.String(), nil
+	result := buf.String()
+	result = strings.ReplaceAll(result, "<no value>", "false")
+
+	return result, nil
 }
 
 func transformOutput(output, transform string, ctx *ExecutionContext) (string, error) {
@@ -299,18 +307,46 @@ func findRecipeSourcesByType(localDir, userDir, publicRepo bool) ([]string, erro
 
 	findYamlFiles := func(root string) ([]string, error) {
 		var files []string
-		err := filepath.Walk(root, func(path string, info os.FileInfo, err error) error {
+		visited := make(map[string]bool)
+
+		var walkDir func(path string) error
+		walkDir = func(path string) error {
+			absPath, err := filepath.Abs(path)
 			if err != nil {
 				return nil
 			}
-			if info.IsDir() {
+
+			if visited[absPath] {
 				return nil
 			}
-			if strings.HasSuffix(path, ".yaml") || strings.HasSuffix(path, ".yml") {
+			visited[absPath] = true
+
+			// Get file info, will follow symlinks
+			fileInfo, err := os.Stat(path)
+			if err != nil {
+				return nil
+			}
+
+			if fileInfo.IsDir() {
+				entries, err := os.ReadDir(path)
+				if err != nil {
+					return nil
+				}
+
+				for _, entry := range entries {
+					entryPath := filepath.Join(path, entry.Name())
+					if err := walkDir(entryPath); err != nil {
+						return err
+					}
+				}
+			} else if strings.HasSuffix(path, ".yaml") || strings.HasSuffix(path, ".yml") {
 				files = append(files, path)
 			}
+
 			return nil
-		})
+		}
+
+		err := walkDir(root)
 		return files, err
 	}
 
@@ -337,113 +373,6 @@ func findRecipeSourcesByType(localDir, userDir, publicRepo bool) ([]string, erro
 	return sources, nil
 }
 
-func syncPublicRecipes() error {
-	if _, err := os.Stat("recipes"); os.IsNotExist(err) {
-		return fmt.Errorf("recipes directory not found - please run this command from the Shef repository root")
-	}
-
-	homeDir, err := os.UserHomeDir()
-	if err != nil {
-		return fmt.Errorf("failed to determine home directory: %w", err)
-	}
-
-	shefDir := filepath.Join(homeDir, ".shef")
-	publicDir := filepath.Join(shefDir, "public")
-	userDir := filepath.Join(shefDir, "user")
-
-	for _, dir := range []string{publicDir, userDir} {
-		if err := os.MkdirAll(dir, 0755); err != nil {
-			return fmt.Errorf("failed to create directory %s: %w", dir, err)
-		}
-	}
-
-	fmt.Println("Copying public recipes...")
-	if err := copyDir("recipes", publicDir); err != nil {
-		return fmt.Errorf("failed to copy recipes: %w", err)
-	}
-
-	fmt.Printf("Public recipes copied to %s\n", publicDir)
-	return nil
-}
-
-func copyDir(src, dst string) error {
-	srcInfo, err := os.Stat(src)
-	if err != nil {
-		return err
-	}
-
-	if err = os.MkdirAll(dst, srcInfo.Mode()); err != nil {
-		return err
-	}
-
-	dir, err := os.Open(src)
-	if err != nil {
-		return err
-	}
-	defer func(dir *os.File) {
-		err := dir.Close()
-		if err != nil {
-
-		}
-	}(dir)
-
-	items, err := dir.Readdir(-1)
-	if err != nil {
-		return err
-	}
-
-	for _, item := range items {
-		srcPath := filepath.Join(src, item.Name())
-		dstPath := filepath.Join(dst, item.Name())
-
-		if item.IsDir() {
-			if err = copyDir(srcPath, dstPath); err != nil {
-				return err
-			}
-		} else {
-			if err = copyFile(srcPath, dstPath); err != nil {
-				return err
-			}
-		}
-	}
-
-	return nil
-}
-
-func copyFile(src, dst string) error {
-	srcFile, err := os.Open(src)
-	if err != nil {
-		return err
-	}
-	defer func(srcFile *os.File) {
-		err := srcFile.Close()
-		if err != nil {
-
-		}
-	}(srcFile)
-
-	dstFile, err := os.Create(dst)
-	if err != nil {
-		return err
-	}
-	defer func(dstFile *os.File) {
-		err := dstFile.Close()
-		if err != nil {
-
-		}
-	}(dstFile)
-
-	if _, err = io.Copy(dstFile, srcFile); err != nil {
-		return err
-	}
-
-	srcInfo, err := os.Stat(src)
-	if err != nil {
-		return err
-	}
-	return os.Chmod(dst, srcInfo.Mode())
-}
-
 func handlePrompt(p Prompt, ctx *ExecutionContext) (interface{}, error) {
 	vars := ctx.templateVars()
 
@@ -814,9 +743,6 @@ func executeCommand(cmdStr string, input string, executionMode string) (string,
 	cmd.Stdin = os.Stdin
 	cmd.Stdout = os.Stdout
 	cmd.Stderr = os.Stderr
-	cmd.SysProcAttr = &syscall.SysProcAttr{
-		Setpgid: false,
-	}
 
 	if err := cmd.Start(); err != nil {
 		return "", fmt.Errorf("failed to start command: %w", err)
@@ -861,6 +787,7 @@ func evaluateCondition(condition string, ctx *ExecutionContext) (bool, error) {
 		if err != nil {
 			return false, fmt.Errorf("failed to render condition template: %w", err)
 		}
+		rendered = strings.ReplaceAll(rendered, "<no value>", "false")
 		return evaluateCondition(rendered, ctx)
 	}
 
@@ -972,6 +899,13 @@ func evaluateNumericComparison(condition string, ctx *ExecutionContext) (bool, e
 		return false, err
 	}
 
+	if leftStr == "false" {
+		leftStr = "0"
+	}
+	if rightStr == "false" {
+		rightStr = "0"
+	}
+
 	leftInt, leftErr := strconv.Atoi(leftStr)
 	rightInt, rightErr := strconv.Atoi(rightStr)
 
@@ -1016,7 +950,7 @@ func evaluateOperationResult(condition string, ctx *ExecutionContext) (bool, err
 			if exists {
 				return result, nil
 			}
-			return false, fmt.Errorf("operation %s result not found", opID)
+			return false, nil
 		}
 	}
 
@@ -1028,7 +962,7 @@ func evaluateOperationResult(condition string, ctx *ExecutionContext) (bool, err
 			if exists {
 				return !result, nil
 			}
-			return false, fmt.Errorf("operation %s result not found", opID)
+			return true, nil
 		}
 	}
 
@@ -1053,7 +987,11 @@ func evaluateVariableComparison(condition string, ctx *ExecutionContext) (bool,
 		return false, fmt.Errorf("invalid variable comparison format: %s", condition)
 	}
 
-	varName := strings.TrimSpace(parts[0])
+	leftPart := strings.TrimSpace(parts[0])
+	rightPart := strings.TrimSpace(parts[1])
+	rightPart = strings.Trim(rightPart, "\"'")
+
+	varName := leftPart
 	if strings.HasPrefix(varName, "$") {
 		varName = varName[1:]
 	}
@@ -1061,25 +999,33 @@ func evaluateVariableComparison(condition string, ctx *ExecutionContext) (bool,
 		varName = varName[1:]
 	}
 
-	expectedValue := strings.TrimSpace(parts[1])
-	expectedValue = strings.Trim(expectedValue, "\"'")
+	var actualValue string
+	var exists bool
 
-	if value, exists := ctx.Vars[varName]; exists {
-		strValue := fmt.Sprintf("%v", value)
-		if op == "==" {
-			return strValue == expectedValue, nil
-		}
-		return strValue != expectedValue, nil
+	if value, ok := ctx.Vars[varName]; ok {
+		actualValue = fmt.Sprintf("%v", value)
+		exists = true
 	}
 
-	if value, exists := ctx.OperationOutputs[varName]; exists {
-		if op == "==" {
-			return value == expectedValue, nil
+	if !exists {
+		if value, ok := ctx.OperationOutputs[varName]; ok {
+			actualValue = value
+			exists = true
 		}
-		return value != expectedValue, nil
 	}
 
-	return false, fmt.Errorf("variable %s not found", varName)
+	if !exists {
+		actualValue = "false"
+	}
+
+	var result bool
+	if op == "==" {
+		result = actualValue == rightPart
+	} else {
+		result = actualValue != rightPart
+	}
+
+	return result, nil
 }
 
 func resolveValue(value string, ctx *ExecutionContext) (string, error) {
@@ -1106,13 +1052,13 @@ func resolveValue(value string, ctx *ExecutionContext) (string, error) {
 		if val, exists := ctx.OperationOutputs[varName]; exists {
 			return val, nil
 		}
-		return "", fmt.Errorf("variable %s not found", varName)
+		return "false", nil
 	}
 
 	return value, nil
 }
 
-func executeRecipe(recipe Recipe, debug bool) error {
+func executeRecipe(recipe Recipe, input string, vars map[string]interface{}, debug bool) error {
 	ctx := ExecutionContext{
 		Data:             "",
 		Vars:             make(map[string]interface{}),
@@ -1120,13 +1066,29 @@ func executeRecipe(recipe Recipe, debug bool) error {
 		OperationResults: make(map[string]bool),
 	}
 
+	for k, v := range vars {
+		ctx.Vars[k] = v
+	}
+
+	if input != "" {
+		ctx.Vars["input"] = input
+		ctx.Data = input
+	}
+
 	opMap := make(map[string]Operation)
 	registerOperations(recipe.Operations, opMap)
 
+	handlerIDs := make(map[string]bool)
+	identifyHandlers(recipe.Operations, handlerIDs)
+
 	if debug {
 		fmt.Println("Registered operations:")
 		for id := range opMap {
-			fmt.Printf("  - %s\n", id)
+			handlerStatus := ""
+			if handlerIDs[id] {
+				handlerStatus = " (handler)"
+			}
+			fmt.Printf("  - %s%s\n", id, handlerStatus)
 		}
 	}
 
@@ -1188,17 +1150,6 @@ func executeRecipe(recipe Recipe, debug bool) error {
 				if debug {
 					fmt.Printf("Skipping operation '%s' (condition not met)\n", op.Name)
 				}
-				if op.ID != "" {
-					ctx.OperationResults[op.ID] = false
-				}
-
-				if op.OnFailure != "" {
-					nextOp, exists := opMap[op.OnFailure]
-					if !exists {
-						return fmt.Errorf("on_failure operation %s not found", op.OnFailure)
-					}
-					return executeOp(nextOp, depth+1)
-				}
 
 				return nil
 			}
@@ -1224,8 +1175,26 @@ func executeRecipe(recipe Recipe, debug bool) error {
 		output, err := executeCommand(cmd, ctx.Data, op.ExecutionMode)
 		operationSuccess := err == nil
 
+		if op.ID != "" {
+			ctx.OperationResults[op.ID] = operationSuccess
+		}
+
 		if err != nil {
-			fmt.Printf("Warning: command execution had errors: %v\n", err)
+			if debug {
+				fmt.Printf("Warning: command execution had errors: %v\n", err)
+			}
+
+			if op.OnFailure != "" {
+				if debug {
+					fmt.Printf("Executing on_failure handler: %s\n", op.OnFailure)
+				}
+
+				nextOp, exists := opMap[op.OnFailure]
+				if !exists {
+					return fmt.Errorf("on_failure operation %s not found", op.OnFailure)
+				}
+				return executeOp(nextOp, depth+1)
+			}
 
 			var continueExecution bool
 			prompt := &survey.Confirm{
@@ -1252,35 +1221,41 @@ func executeRecipe(recipe Recipe, debug bool) error {
 			}
 		}
 
+		ctx.Data = output
+
 		if op.ID != "" {
-			ctx.OperationResults[op.ID] = operationSuccess
 			ctx.OperationOutputs[op.ID] = strings.TrimSpace(output)
 		}
 
-		ctx.Data = output
-
 		if output != "" && !op.Silent {
 			fmt.Println(output)
 		}
 
-		if op.OnSuccess != "" && ctx.OperationResults[op.ID] {
+		if op.OnSuccess != "" && operationSuccess {
 			nextOp, exists := opMap[op.OnSuccess]
 			if !exists {
 				return fmt.Errorf("on_success operation %s not found", op.OnSuccess)
 			}
 			return executeOp(nextOp, depth+1)
-		} else if op.OnFailure != "" && !ctx.OperationResults[op.ID] {
-			nextOp, exists := opMap[op.OnFailure]
-			if !exists {
-				return fmt.Errorf("on_failure operation %s not found", op.OnFailure)
-			}
-			return executeOp(nextOp, depth+1)
+		}
+
+		if debug {
+			fmt.Printf("Operation %s result: %v\n", op.ID, ctx.OperationResults[op.ID])
+			fmt.Printf("Handler for on_success: '%s'\n", op.OnSuccess)
+			fmt.Printf("Handler for on_failure: '%s'\n", op.OnFailure)
 		}
 
 		return nil
 	}
 
 	for i, op := range recipe.Operations {
+		if op.ID != "" && handlerIDs[op.ID] {
+			if debug {
+				fmt.Printf("Skipping handler operation %d: %s (ID: %s)\n", i+1, op.Name, op.ID)
+			}
+			continue
+		}
+
 		if debug {
 			fmt.Printf("Executing operation %d: %s\n", i+1, op.Name)
 		}
@@ -1288,10 +1263,6 @@ func executeRecipe(recipe Recipe, debug bool) error {
 		if err := executeOp(op, 0); err != nil {
 			return err
 		}
-
-		if op.OnSuccess != "" || op.OnFailure != "" {
-			break
-		}
 	}
 
 	return nil
@@ -1309,6 +1280,21 @@ func registerOperations(operations []Operation, opMap map[string]Operation) {
 	}
 }
 
+func identifyHandlers(operations []Operation, handlerIDs map[string]bool) {
+	for _, op := range operations {
+		if op.OnSuccess != "" {
+			handlerIDs[op.OnSuccess] = true
+		}
+		if op.OnFailure != "" {
+			handlerIDs[op.OnFailure] = true
+		}
+
+		if op.ControlFlow != nil && len(op.Operations) > 0 {
+			identifyHandlers(op.Operations, handlerIDs)
+		}
+	}
+}
+
 func listRecipes(recipes []Recipe) {
 	if len(recipes) == 0 {
 		fmt.Println("No recipes found.")
@@ -1359,7 +1345,7 @@ func main() {
 	app := &cli.App{
 		Name:    "shef",
 		Usage:   "Shef is a powerful CLI tool that lets you combine shell commands into reusable recipes.",
-		Version: "0.1.0",
+		Version: Version,
 		Flags: []cli.Flag{
 			&cli.BoolFlag{
 				Name:    "debug",
@@ -1387,6 +1373,11 @@ func main() {
 				Aliases: []string{"c"},
 				Usage:   "Filter by category",
 			},
+			&cli.PathFlag{
+				Name:    "recipe-file",
+				Aliases: []string{"r"},
+				Usage:   "Path to the recipe file",
+			},
 		},
 		Action: func(c *cli.Context) error {
 			args := c.Args().Slice()
@@ -1514,32 +1505,92 @@ func outputRecipesAsJSON(recipes []Recipe) error {
 }
 
 func handleRunCommand(c *cli.Context, args []string, sourcePriority []string) error {
-	if len(args) == 0 {
-		return fmt.Errorf("no recipe specified. Use shef ls to list available recipes")
-	}
-
-	var category, recipeName string
-	if len(args) == 1 {
-		recipeName = args[0]
-		category = c.String("category")
-	} else {
-		category = args[0]
-		recipeName = args[1]
-	}
-
-	recipe, err := findRecipeInSources(recipeName, category, sourcePriority)
-	if err != nil {
-		return err
-	}
-
 	debug := c.Bool("debug")
+	var recipes []Recipe
+	var remainingArgs []string
 
-	if debug {
-		fmt.Printf("Running recipe: %s\n", recipe.Name)
-		fmt.Printf("Description: %s\n\n", recipe.Description)
+	recipeFilePath := c.String("recipe-file")
+	if recipeFilePath != "" {
+		config, err := loadConfig(recipeFilePath)
+		if err != nil {
+			return fmt.Errorf("failed to load recipe file %s: %w", recipeFilePath, err)
+		}
+		recipes = config.Recipes
+		remainingArgs = args
+	} else {
+		if len(args) == 0 {
+			return fmt.Errorf("no recipe specified. Use shef ls to list available recipes")
+		}
+
+		recipe, err := findRecipeInSources(args[0], "", sourcePriority)
+
+		if err != nil && len(args) > 1 {
+			recipe, err = findRecipeInSources(args[1], args[0], sourcePriority)
+			if err == nil {
+				remainingArgs = args[2:]
+			}
+		} else if err == nil {
+			remainingArgs = args[1:]
+		}
+
+		if err != nil {
+			return err
+		}
+
+		recipes = []Recipe{*recipe}
 	}
 
-	return executeRecipe(*recipe, debug)
+	input, vars := processRemainingArgs(remainingArgs)
+
+	for _, recipe := range recipes {
+		if debug {
+			fmt.Printf("Running recipe: %s\n", recipe.Name)
+			fmt.Printf("With input: %s\n", input)
+			fmt.Printf("With vars: %v\n", vars)
+			fmt.Printf("Description: %s\n\n", recipe.Description)
+		}
+
+		if err := executeRecipe(recipe, input, vars, debug); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func processRemainingArgs(args []string) (string, map[string]interface{}) {
+	vars := make(map[string]interface{})
+	var input string
+
+	for _, arg := range args {
+		if strings.HasPrefix(arg, "-") {
+			if strings.HasPrefix(arg, "--") {
+				arg = arg[2:] // Remove --
+				if strings.Contains(arg, "=") {
+					parts := strings.SplitN(arg, "=", 2)
+					flagName := strings.ReplaceAll(parts[0], "-", "_")
+					vars[flagName] = parts[1]
+				} else {
+					flagName := strings.ReplaceAll(arg, "-", "_")
+					vars[flagName] = true
+				}
+			} else {
+				arg = arg[1:] // Remove -
+				if strings.Contains(arg, "=") {
+					parts := strings.SplitN(arg, "=", 2)
+					vars[parts[0]] = parts[1]
+				} else {
+					for _, c := range arg {
+						vars[string(c)] = true
+					}
+				}
+			}
+		} else if input == "" {
+			input = arg
+		}
+	}
+
+	return input, vars
 }
 
 func findRecipeInSources(recipeName, category string, sourcePriority []string) (*Recipe, error) {
